/*
The follwoing program code generates the approximated tail lower bound distribution of the 
QLR tet statistic given in the papger, Testing for Regime Switching by Cho, JS and White, H. 

The first step to obtain this bound is to compute 

    integral_{Theta} gamma(theta) d_theta

We compute this by the numerical integration method, and next obtain the desired lower bound
by Theorem 8. The following four different cases are those considered in Table 1.

Case 1: When the variance term is known, and the null parameter value is at the center of 
        the parameter space. 

        The matrix "ttc" given below provides the desired lower bound. The first column of 
        "ttc" represent the percentile, and the other columns are the qunatiles corresponding 
        to the various paraemeter spaces considered in the paper. For example, ttc[.,2] 
        provides the tail lower bound when the parameter space is [-1, 1].

Case 2: When the variance term is known, and the null parameter value is at the corner of 
        the parameter space. 

        The matrix "ttc0" given below provides the desired lower bound. The first column of
        "ttc0" represents the percentile, and the other columns are the qunatiles corresponding 
        to the various paraemeter spaces consdiered in the paper. For example, ttc0[.,2] 
        provides the tail lower bound when the parameter space is [0, 1].

Case 3: When the variance term is unknown, and the null parameter value is at the center of 
        the parameter space. 

        The matrix "ttd" given below provides the desired lower bound. The first column of 
        "ttd" represents the percentile, and the other columns are the qunatiles corresponding 
        to the various paraemeter spaces considered in the paper. For example, ttd[.,2] 
        provides the tail lower bound when the parameter space is [-1, 1].

Case 4: When the variance term is unknown, and the null parameter value is at the corner of 
        the parameter space. 

        The matrix "ttd0" given below provides the desired lower bound. The first column of
        "ttd0" represents the percentile, and the other columns are the qunatiles corresponding 
        to the various paraemeter spaces consdiered in the paper. For example, ttd0[.,2] 
        provides the tail lower bound when the parameter space is [0, 1].

Comment: Researchers can modify the parameter space, and also they need to modify the functional
form of gamma(theta) if they consider different mixture distributions.
*/

new;
library pgraph;

_intord = 40;

proc rc11(x); 
    retp(sqrt( (1+exp(2*x^2)-exp(x^2).*(2+x^4))./(1-exp(x^2)+x^2)^2 )); 
endp;

proc rd11(x); 
    retp(sqrt( 2*(2+2*exp(2*x^2)+4*x^2+x^4-exp(x^2).*(4+4*x^2-x^4+x^6))
               ./(2-2*exp(x^2)+2*x^2+x^4)^2 )); 
endp;

uuu = seqa(0, 10/10000, 10001);
ttc = zeros(10001,6);
ttc0= zeros(10001,6);
ttd = zeros(10001,6);
ttd0= zeros(10001,6);

/* parameter space [-1, 1] */
x11 = 1|-1; 

/* parameter space [-2, 2] */
x22 = 2|-2;

/* parameter space [-3, 3] */
x33 = 3|-3;

/* parameter space [-4, 4] */
x44 = 4|-4;

/* parameter space [-5, 5] */
x55 = 5|-5;

c11 = intquad1(&rc11, x11);
d11 = intquad1(&rd11, x11);
c01 = c11/2;
d01 = d11/2;

c22 = intquad1(&rc11, x22);
d22 = intquad1(&rd11, x22);
c02 = c22/2;
d02 = d22/2;

c33 = intquad1(&rc11, x33);
d33 = intquad1(&rd11, x33);
c03 = c33/2;
d03 = d33/2;

c44 = intquad1(&rc11, x44);
d44 = intquad1(&rd11, x44);
c04 = c44/2;
d04 = d44/2;

c55 = intquad1(&rc11, x55);
d55 = intquad1(&rd11, x55);
c05 = c55/2;
d05 = d55/2;

iii = 1;
do until iii > 10001;
    ttc[iii,1] = sqrt(uuu[iii,1]);
    ttc[iii,2] = 1 - 2*( pdfn(ttc[iii,1])*c01/sqrt(2*pi) + (1-cdfn(ttc[iii,1])) );
    ttc[iii,3] = 1 - 2*( pdfn(ttc[iii,1])*c02/sqrt(2*pi) + (1-cdfn(ttc[iii,1])) );
    ttc[iii,4] = 1 - 2*( pdfn(ttc[iii,1])*c03/sqrt(2*pi) + (1-cdfn(ttc[iii,1])) );
    ttc[iii,5] = 1 - 2*( pdfn(ttc[iii,1])*c04/sqrt(2*pi) + (1-cdfn(ttc[iii,1])) );
    ttc[iii,6] = 1 - 2*( pdfn(ttc[iii,1])*c05/sqrt(2*pi) + (1-cdfn(ttc[iii,1])) );

    ttc0[iii,1] = sqrt(uuu[iii,1]);
    ttc0[iii,2] = 1 - ( pdfn(ttc0[iii,1])*c01/sqrt(2*pi)+(1-cdfn(ttc0[iii,1])) );
    ttc0[iii,3] = 1 - ( pdfn(ttc0[iii,1])*c02/sqrt(2*pi)+(1-cdfn(ttc0[iii,1])) );
    ttc0[iii,4] = 1 - ( pdfn(ttc0[iii,1])*c03/sqrt(2*pi)+(1-cdfn(ttc0[iii,1])) );
    ttc0[iii,5] = 1 - ( pdfn(ttc0[iii,1])*c04/sqrt(2*pi)+(1-cdfn(ttc0[iii,1])) );
    ttc0[iii,6] = 1 - ( pdfn(ttc0[iii,1])*c05/sqrt(2*pi)+(1-cdfn(ttc0[iii,1])) );

    ttd[iii,1] = sqrt(uuu[iii,1]);
    ttd[iii,2] = 1 - 2*( pdfn(ttd[iii,1])*d01/sqrt(2*pi)+(1-cdfn(ttd[iii,1])) ) - (1-cdfn(ttd[iii,1]));
    ttd[iii,3] = 1 - 2*( pdfn(ttd[iii,1])*d02/sqrt(2*pi)+(1-cdfn(ttd[iii,1])) ) - (1-cdfn(ttd[iii,1]));
    ttd[iii,4] = 1 - 2*( pdfn(ttd[iii,1])*d03/sqrt(2*pi)+(1-cdfn(ttd[iii,1])) ) - (1-cdfn(ttd[iii,1]));
    ttd[iii,5] = 1 - 2*( pdfn(ttd[iii,1])*d04/sqrt(2*pi)+(1-cdfn(ttd[iii,1])) ) - (1-cdfn(ttd[iii,1]));
    ttd[iii,6] = 1 - 2*( pdfn(ttd[iii,1])*d05/sqrt(2*pi)+(1-cdfn(ttd[iii,1])) ) - (1-cdfn(ttd[iii,1]));

    ttd0[iii,1] = sqrt(uuu[iii,1]);
    ttd0[iii,2] = 1 - ( pdfn(ttd0[iii,1])*d01/sqrt(2*pi)+(1-cdfn(ttd0[iii,1])) ) - (1-cdfn(ttd0[iii,1]));
    ttd0[iii,3] = 1 - ( pdfn(ttd0[iii,1])*d02/sqrt(2*pi)+(1-cdfn(ttd0[iii,1])) ) - (1-cdfn(ttd0[iii,1]));
    ttd0[iii,4] = 1 - ( pdfn(ttd0[iii,1])*d03/sqrt(2*pi)+(1-cdfn(ttd0[iii,1])) ) - (1-cdfn(ttd0[iii,1]));
    ttd0[iii,5] = 1 - ( pdfn(ttd0[iii,1])*d04/sqrt(2*pi)+(1-cdfn(ttd0[iii,1])) ) - (1-cdfn(ttd0[iii,1]));
    ttd0[iii,6] = 1 - ( pdfn(ttd0[iii,1])*d05/sqrt(2*pi)+(1-cdfn(ttd0[iii,1])) ) - (1-cdfn(ttd0[iii,1]));

    iii = iii + 1;
endo;

print "Case 1";
print uuu~ttc[.,2:6];

print "Case 2";
print uuu~ttc0[.,2:6];

print "Case 3";
print uuu~ttd[.,2:6];

print "Case 4";
print uuu~ttd0[.,2:6];
